package flash.text;

extern class TextExtent {
	var ascent : Float;
	var descent : Float;
	var height : Float;
	var textFieldHeight : Float;
	var textFieldWidth : Float;
	var width : Float;
	function new(width : Float, height : Float, textFieldWidth : Float, textFieldHeight : Float, ascent : Float, descent : Float) : Void;
}
