package flash.media;

extern class AVCaptionStyle {
	@:flash.property var backgroundColor(get,set) : String;
	@:flash.property var backgroundOpacity(get,set) : String;
	@:flash.property var bottomInset(get,set) : String;
	@:flash.property var edgeColor(get,set) : String;
	@:flash.property var fillColor(get,set) : String;
	@:flash.property var fillOpacity(get,set) : String;
	@:flash.property var font(get,set) : String;
	@:flash.property var fontColor(get,set) : String;
	@:flash.property var fontEdge(get,set) : String;
	@:flash.property var fontOpacity(get,set) : String;
	@:flash.property var size(get,set) : String;
	function new() : Void;
	private function get_backgroundColor() : String;
	private function get_backgroundOpacity() : String;
	private function get_bottomInset() : String;
	private function get_edgeColor() : String;
	private function get_fillColor() : String;
	private function get_fillOpacity() : String;
	private function get_font() : String;
	private function get_fontColor() : String;
	private function get_fontEdge() : String;
	private function get_fontOpacity() : String;
	private function get_size() : String;
	private function set_backgroundColor(value : String) : String;
	private function set_backgroundOpacity(value : String) : String;
	private function set_bottomInset(value : String) : String;
	private function set_edgeColor(value : String) : String;
	private function set_fillColor(value : String) : String;
	private function set_fillOpacity(value : String) : String;
	private function set_font(value : String) : String;
	private function set_fontColor(value : String) : String;
	private function set_fontEdge(value : String) : String;
	private function set_fontOpacity(value : String) : String;
	private function set_size(value : String) : String;
	static final BLACK : String;
	static final BLUE : String;
	static final BRIGHT_BLUE : String;
	static final BRIGHT_CYAN : String;
	static final BRIGHT_GREEN : String;
	static final BRIGHT_MAGENTA : String;
	static final BRIGHT_RED : String;
	static final BRIGHT_WHITE : String;
	static final BRIGHT_YELLOW : String;
	static final CASUAL : String;
	static final CURSIVE : String;
	static final CYAN : String;
	static final DARK_BLUE : String;
	static final DARK_CYAN : String;
	static final DARK_GREEN : String;
	static final DARK_MAGENTA : String;
	static final DARK_RED : String;
	static final DARK_YELLOW : String;
	static final DEFAULT : String;
	static final DEPRESSED : String;
	static final GRAY : String;
	static final GREEN : String;
	static final LARGE : String;
	static final LEFT_DROP_SHADOW : String;
	static final MAGENTA : String;
	static final MEDIUM : String;
	static final MONOSPACED_WITHOUT_SERIFS : String;
	static final MONOSPACE_WITH_SERIFS : String;
	static final NONE : String;
	static final PROPORTIONAL_WITHOUT_SERIFS : String;
	static final PROPORTIONAL_WITH_SERIFS : String;
	static final RAISED : String;
	static final RED : String;
	static final RIGHT_DROP_SHADOW : String;
	static final SMALL : String;
	static final SMALL_CAPITALS : String;
	static final UNIFORM : String;
	static final WHITE : String;
	static final YELLOW : String;
}
