package flash.media;

extern class AudioDecoder {
	function new() : Void;
	static final DOLBY_DIGITAL : String;
	static final DOLBY_DIGITAL_PLUS : String;
	static final DTS : String;
	static final DTS_EXPRESS : String;
	static final DTS_HD_HIGH_RESOLUTION_AUDIO : String;
	static final DTS_HD_MASTER_AUDIO : String;
}
