package flash.sampler;

extern class ClassFactory {
	function new() : Void;
	static final DeleteObjectSampleClass : Class<Dynamic>;
	static final NewObjectSampleClass : Class<Dynamic>;
	static final SampleClass : Class<Dynamic>;
	static final StackFrameClass : Class<Dynamic>;
}
