package flash.display3D;

@:native("flash.display3D.Context3DProfile") extern enum abstract Context3DProfile(String) {
	var BASELINE;
	var BASELINE_CONSTRAINED;
	var BASELINE_EXTENDED;
	var STANDARD;
	var STANDARD_CONSTRAINED;
	var STANDARD_EXTENDED;
}
