package flash.net;

extern class URLRequestMethod {
	@:require(flash10_1) static final DELETE : String;
	static final GET : String;
	@:require(flash10_1) static final HEAD : String;
	@:require(flash10_1) static final OPTIONS : String;
	static final POST : String;
	@:require(flash10_1) static final PUT : String;
}
