package flash.text.engine;

@:native("flash.text.engine.TextBaseline") extern enum abstract TextBaseline(String) {
	var ASCENT;
	var DESCENT;
	var IDEOGRAPHIC_BOTTOM;
	var IDEOGRAPHIC_CENTER;
	var IDEOGRAPHIC_TOP;
	var ROMAN;
	var USE_DOMINANT_BASELINE;
}
