package flash.text.engine;

@:native("flash.text.engine.TypographicCase") extern enum abstract TypographicCase(String) {
	var CAPS;
	var CAPS_AND_SMALL_CAPS;
	var DEFAULT;
	var LOWERCASE;
	var SMALL_CAPS;
	var TITLE;
	var UPPERCASE;
}
