/*
 * Copyright (C)2005-2019 Haxe Foundation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

// This file is generated from mozilla\StyleSheet.webidl. Do not edit!

package js.html;

/**
	An object implementing the `StyleSheet` interface represents a single style sheet. CSS style sheets will further implement the more specialized `CSSStyleSheet` interface.

	Documentation [StyleSheet](https://developer.mozilla.org/en-US/docs/Web/API/StyleSheet) by [Mozilla Contributors](https://developer.mozilla.org/en-US/docs/Web/API/StyleSheet$history), licensed under [CC-BY-SA 2.5](https://creativecommons.org/licenses/by-sa/2.5/).

	@see <https://developer.mozilla.org/en-US/docs/Web/API/StyleSheet>
**/
@:native("StyleSheet")
extern class StyleSheet {
	
	/**
		Returns a `DOMString` representing the style sheet language for this style sheet.
	**/
	var type(default,null) : String;
	
	/**
		Returns a `DOMString` representing the location of the stylesheet.
	**/
	var href(default,null) : String;
	
	/**
		Returns a `Node` associating this style sheet with the current document.
	**/
	var ownerNode(default,null) : Node;
	
	/**
		Returns a `StyleSheet` including this one, if any; returns `null` if there aren't any.
	**/
	var parentStyleSheet(default,null) : StyleSheet;
	
	/**
		Returns a `DOMString` representing the advisory title of the current style sheet.
	**/
	var title(default,null) : String;
	
	/**
		Returns a `MediaList` representing the intended destination medium for style information.
	**/
	var media(default,null) : MediaList;
	
	/**
		Is a `Boolean` representing whether the current stylesheet has been applied or not.
	**/
	var disabled : Bool;
	
}