package flash.globalization;

@:require(flash10_1) extern final class NumberFormatter {
	@:flash.property var actualLocaleIDName(get,never) : String;
	@:flash.property var decimalSeparator(get,set) : String;
	@:flash.property var digitsType(get,set) : NationalDigitsType;
	@:flash.property var fractionalDigits(get,set) : Int;
	@:flash.property var groupingPattern(get,set) : String;
	@:flash.property var groupingSeparator(get,set) : String;
	@:flash.property var lastOperationStatus(get,never) : LastOperationStatus;
	@:flash.property var leadingZero(get,set) : Bool;
	@:flash.property var negativeNumberFormat(get,set) : UInt;
	@:flash.property var negativeSymbol(get,set) : String;
	@:flash.property var requestedLocaleIDName(get,never) : String;
	@:flash.property var trailingZeros(get,set) : Bool;
	@:flash.property var useGrouping(get,set) : Bool;
	function new(requestedLocaleIDName : String) : Void;
	function formatInt(value : Int) : String;
	function formatNumber(value : Float) : String;
	function formatUint(value : UInt) : String;
	private function get_actualLocaleIDName() : String;
	private function get_decimalSeparator() : String;
	private function get_digitsType() : NationalDigitsType;
	private function get_fractionalDigits() : Int;
	private function get_groupingPattern() : String;
	private function get_groupingSeparator() : String;
	private function get_lastOperationStatus() : LastOperationStatus;
	private function get_leadingZero() : Bool;
	private function get_negativeNumberFormat() : UInt;
	private function get_negativeSymbol() : String;
	private function get_requestedLocaleIDName() : String;
	private function get_trailingZeros() : Bool;
	private function get_useGrouping() : Bool;
	function parse(parseString : String) : NumberParseResult;
	function parseNumber(parseString : String) : Float;
	private function set_decimalSeparator(value : String) : String;
	private function set_digitsType(value : NationalDigitsType) : NationalDigitsType;
	private function set_fractionalDigits(value : Int) : Int;
	private function set_groupingPattern(value : String) : String;
	private function set_groupingSeparator(value : String) : String;
	private function set_leadingZero(value : Bool) : Bool;
	private function set_negativeNumberFormat(value : UInt) : UInt;
	private function set_negativeSymbol(value : String) : String;
	private function set_trailingZeros(value : Bool) : Bool;
	private function set_useGrouping(value : Bool) : Bool;
	static function getAvailableLocaleIDNames() : flash.Vector<String>;
}
