package flash.ui;

extern final class ContextMenuBuiltInItems {
	@:flash.property var forwardAndBack(get,set) : Bool;
	@:flash.property var loop(get,set) : Bool;
	@:flash.property var play(get,set) : Bool;
	@:flash.property var print(get,set) : Bool;
	@:flash.property var quality(get,set) : Bool;
	@:flash.property var rewind(get,set) : Bool;
	@:flash.property var save(get,set) : Bool;
	@:flash.property var zoom(get,set) : Bool;
	function new() : Void;
	function clone() : ContextMenuBuiltInItems;
	private function get_forwardAndBack() : Bool;
	private function get_loop() : Bool;
	private function get_play() : Bool;
	private function get_print() : Bool;
	private function get_quality() : Bool;
	private function get_rewind() : Bool;
	private function get_save() : Bool;
	private function get_zoom() : Bool;
	private function set_forwardAndBack(value : Bool) : Bool;
	private function set_loop(value : Bool) : Bool;
	private function set_play(value : Bool) : Bool;
	private function set_print(value : Bool) : Bool;
	private function set_quality(value : Bool) : Bool;
	private function set_rewind(value : Bool) : Bool;
	private function set_save(value : Bool) : Bool;
	private function set_zoom(value : Bool) : Bool;
}
