package flash.display;

@:native("flash.display.BlendMode") extern enum abstract BlendMode(String) {
	var ADD;
	var ALPHA;
	var DARKEN;
	var DIFFERENCE;
	var ERASE;
	var HARDLIGHT;
	var INVERT;
	var LAYER;
	var LIGHTEN;
	var MULTIPLY;
	var NORMAL;
	var OVERLAY;
	var SCREEN;
	var SHADER;
	var SUBTRACT;
}
