package flash.display;

@:native("flash.display.ShaderParameterType") extern enum abstract ShaderParameterType(String) {
	var BOOL;
	var BOOL2;
	var BOOL3;
	var BOOL4;
	var FLOAT;
	var FLOAT2;
	var FLOAT3;
	var FLOAT4;
	var INT;
	var INT2;
	var INT3;
	var INT4;
	var MATRIX2X2;
	var MATRIX3X3;
	var MATRIX4X4;
}
