package flash.text;

extern class TextFormat {
	@:flash.property var align(get,set) : TextFormatAlign;
	@:flash.property var blockIndent(get,set) : Null<Float>;
	@:flash.property var bold(get,set) : Null<Bool>;
	@:flash.property var bullet(get,set) : Null<Bool>;
	@:flash.property var color(get,set) : Null<UInt>;
	@:flash.property var display(get,set) : TextFormatDisplay;
	@:flash.property var font(get,set) : String;
	@:flash.property var indent(get,set) : Null<Float>;
	@:flash.property var italic(get,set) : Null<Bool>;
	@:flash.property var kerning(get,set) : Null<Bool>;
	@:flash.property var leading(get,set) : Null<Float>;
	@:flash.property var leftMargin(get,set) : Null<Float>;
	@:flash.property var letterSpacing(get,set) : Null<Float>;
	@:flash.property var rightMargin(get,set) : Null<Float>;
	@:flash.property var size(get,set) : Null<Float>;
	@:flash.property var tabStops(get,set) : Array<UInt>;
	@:flash.property var target(get,set) : String;
	@:flash.property var underline(get,set) : Null<Bool>;
	@:flash.property var url(get,set) : String;
	function new(?font : String, size : Null<Float> = 0, color : Null<UInt> = 0, bold : Null<Bool> = false, italic : Null<Bool> = false, underline : Null<Bool> = false, ?url : String, ?target : String, ?align : TextFormatAlign, leftMargin : Null<Float> = 0, rightMargin : Null<Float> = 0, indent : Null<Float> = 0, leading : Null<Float> = 0) : Void;
	private function get_align() : TextFormatAlign;
	private function get_blockIndent() : Null<Float>;
	private function get_bold() : Null<Bool>;
	private function get_bullet() : Null<Bool>;
	private function get_color() : Null<UInt>;
	private function get_display() : TextFormatDisplay;
	private function get_font() : String;
	private function get_indent() : Null<Float>;
	private function get_italic() : Null<Bool>;
	private function get_kerning() : Null<Bool>;
	private function get_leading() : Null<Float>;
	private function get_leftMargin() : Null<Float>;
	private function get_letterSpacing() : Null<Float>;
	private function get_rightMargin() : Null<Float>;
	private function get_size() : Null<Float>;
	private function get_tabStops() : Array<UInt>;
	private function get_target() : String;
	private function get_underline() : Null<Bool>;
	private function get_url() : String;
	private function set_align(value : TextFormatAlign) : TextFormatAlign;
	private function set_blockIndent(value : Null<Float>) : Null<Float>;
	private function set_bold(value : Null<Bool>) : Null<Bool>;
	private function set_bullet(value : Null<Bool>) : Null<Bool>;
	private function set_color(value : Null<UInt>) : Null<UInt>;
	private function set_display(value : TextFormatDisplay) : TextFormatDisplay;
	private function set_font(value : String) : String;
	private function set_indent(value : Null<Float>) : Null<Float>;
	private function set_italic(value : Null<Bool>) : Null<Bool>;
	private function set_kerning(value : Null<Bool>) : Null<Bool>;
	private function set_leading(value : Null<Float>) : Null<Float>;
	private function set_leftMargin(value : Null<Float>) : Null<Float>;
	private function set_letterSpacing(value : Null<Float>) : Null<Float>;
	private function set_rightMargin(value : Null<Float>) : Null<Float>;
	private function set_size(value : Null<Float>) : Null<Float>;
	private function set_tabStops(value : Array<UInt>) : Array<UInt>;
	private function set_target(value : String) : String;
	private function set_underline(value : Null<Bool>) : Null<Bool>;
	private function set_url(value : String) : String;
}
