package flash.display3D;

@:native("flash.display3D.Context3DVertexBufferFormat") extern enum abstract Context3DVertexBufferFormat(String) {
	var BYTES_4;
	var FLOAT_1;
	var FLOAT_2;
	var FLOAT_3;
	var FLOAT_4;
}
