package flash.media;

extern class AVResult {
	@:flash.property var result(get,never) : Int;
	function new(inResult : Int) : Void;
	private function get_result() : Int;
	static final ASYNC_OPERATION_IN_PROGRESS : Int;
	static final AUDIO_ONLY_STREAM_END : Int;
	static final AUDIO_ONLY_STREAM_START : Int;
	static final AUDIO_START_ERROR : Int;
	static final BAD_MANIFEST_SIGNATURE : Int;
	static final BAD_MEDIASAMPLE_FOUND : Int;
	static final BAD_MEDIA_INTERLEAVING : Int;
	static final CALLED_FROM_WRONG_THREAD : Int;
	static final CANNOT_ERASE_TIMELINE : Int;
	static final CANNOT_FAIL_OVER : Int;
	static final CANNOT_HANDLE_MAIN_MANIFEST_UPDATE : Int;
	static final CANNOT_LOAD_PLAY_LIST : Int;
	static final CANNOT_SPLIT_TIMELINE : Int;
	static final CODEC_NOT_SUPPORTED : Int;
	static final COMPONENT_CREATION_FAILURE : Int;
	static final CONTAINER_NOT_SUPPORTED : Int;
	static final CONTENT_LENGTH_MISMATCH : Int;
	static final CRYPTO_ALGORITHM_NOT_SUPPORTED : Int;
	static final CRYPTO_ERROR_BAD_CERTIFICATE : Int;
	static final CRYPTO_ERROR_BAD_PARAMETER : Int;
	static final CRYPTO_ERROR_BUFFER_TOO_SMALL : Int;
	static final CRYPTO_ERROR_CORRUPTED_DATA : Int;
	static final CRYPTO_ERROR_DIGEST_FINISH : Int;
	static final CRYPTO_ERROR_DIGEST_UPDATE : Int;
	static final CRYPTO_ERROR_UNKNOWN : Int;
	static final CURRENT_PERIOD_EXPIRED : Int;
	static final DECODER_FAILED : Int;
	static final DEVICE_OPEN_ERROR : Int;
	static final DID_NOT_GET_NEXT_FRAGMENT : Int;
	static final DRM_INIT_ERROR : Int;
	static final DRM_NOT_AVAILABLE : Int;
	static final END_OF_PERIOD : Int;
	static final EOF : Int;
	static final FILE_NOT_FOUND : Int;
	static final FILE_OPEN_ERROR : Int;
	static final FILE_READ_ERROR : Int;
	static final FILE_STRUCTURE_INVALID : Int;
	static final FILE_WRITE_ERROR : Int;
	static final FRAGMENT_READ_ERROR : Int;
	static final GENERIC_ERROR : Int;
	static final HTTP_TIME_OUT : Int;
	static final ID3_PARSE_ERROR : Int;
	static final INCOMPATIBLE_RENDER_MODE : Int;
	static final INCOMPATIBLE_VERSION : Int;
	static final INTERNAL_ERROR : Int;
	static final INVALID_KEY : Int;
	static final INVALID_OPERATION : Int;
	static final INVALID_PARAMETER : Int;
	static final INVALID_REPLACE_DURATION : Int;
	static final INVALID_SEEK_TIME : Int;
	static final INVALID_WITH_AUDIO_ONLY_FILE : Int;
	static final IRRECOVERABLE_ERROR : Int;
	static final KEY_NOT_FOUND : Int;
	static final KEY_SERVER_NOT_FOUND : Int;
	static final LISTENER_NOT_FOUND : Int;
	static final LIVE_HOLD : Int;
	static final LIVE_WINDOW_MOVED_BACKWARD : Int;
	static final LOST_CONNECTION_RECOVERABLE : Int;
	static final MAIN_MANIFEST_UPDATE_TO_BE_HANDLED : Int;
	static final MANIFEST_FILE_UNEXPECTEDLY_CHANGED : Int;
	static final NETWORK_DOWN : Int;
	static final NETWORK_ERROR : Int;
	static final NETWORK_UNAVAILABLE : Int;
	static final NOT_IMPLEMENTED : Int;
	static final NO_AUDIO_SINK : Int;
	static final NO_FIXED_SIZE : Int;
	static final NO_TIMELINE : Int;
	static final NO_USEABLE_BITRATE_PROFILE : Int;
	static final NULL_OPERATION : Int;
	static final ONLY_ALLOWED_IN_PAUSED_STATE : Int;
	static final OPERATION_ABORTED : Int;
	static final OUT_OF_MEMORY : Int;
	static final OVERFLOW : Int;
	static final PARSE_ERROR : Int;
	static final PARTIAL_REPLACEMENT : Int;
	static final PERIOD_HOLD : Int;
	static final PERIOD_NOT_LOADED : Int;
	static final PLAYBACK_NOT_ENABLED : Int;
	static final POSTROLL_WITH_LIVE_NOT_ALLOWED : Int;
	static final PREVIOUS_STEP_SEEK_IN_PROGRESS : Int;
	static final PROTOCOL_NOT_SUPPORTED : Int;
	static final RANGE_ERROR : Int;
	static final RANGE_SPANS_READHEAD : Int;
	static final RENDITION_M3U8_ERROR : Int;
	static final REPLACEMENT_FAILED : Int;
	static final RESOURCE_NOT_SPECIFIED : Int;
	static final SECURITY_ERROR : Int;
	static final SEEK_FAILED : Int;
	static final SEGMENT_SKIPPED_ON_FAILURE : Int;
	static final SIZE_UNKNOWN : Int;
	static final SPS_PPS_FOUND_OUTSIDE_AVCC : Int;
	static final SUCCESS : Int;
	static final SWITCH_TO_ASYMMETRIC_PROFILE : Int;
	static final TIMELINE_TOO_SHORT : Int;
	static final UNDERFLOW : Int;
	static final UNREPORTED_TIME_DISCONTINUITY_FOUND : Int;
	static final UNSUPPORTED_CONFIGURATION : Int;
	static final UNSUPPORTED_HLS_VERSION : Int;
	static final UNSUPPORTED_OPERATION : Int;
	static final VIDEO_PROFILE_NOT_SUPPORTED : Int;
	static final WAITING_FOR_INIT : Int;
}
