package flash.display;

@:native("flash.display.StageAlign") extern enum abstract StageAlign(String) {
	var BOTTOM;
	var BOTTOM_LEFT;
	var BOTTOM_RIGHT;
	var LEFT;
	var RIGHT;
	var TOP;
	var TOP_LEFT;
	var TOP_RIGHT;
}
