package flash.ui;

@:native("flash.ui.GameInputControlType") extern enum abstract GameInputControlType(String) {
	var ACCELERATION;
	var BUTTON;
	var DIRECTION;
	var MOVEMENT;
	var ROTATION;
	var TRIGGER;
}
