package flash.text;

extern final class StaticText extends flash.display.DisplayObject {
	@:flash.property var text(get,never) : String;
	function new() : Void;
	private function get_text() : String;
}
