package flash.display;

@:native("flash.display.BitmapEncodingColorSpace") extern enum abstract BitmapEncodingColorSpace(String) {
	var COLORSPACE_4_2_0;
	var COLORSPACE_4_2_2;
	var COLORSPACE_4_4_4;
	var COLORSPACE_AUTO;
}
