package flash.ui;

extern class Keyboard {
	@:require(flash10_1) static final A : UInt;
	@:require(flash10_1) static final ALTERNATE : UInt;
	@:require(flash10_2) static final AUDIO : UInt;
	@:require(flash10_1) static final B : UInt;
	@:require(flash10_2) static final BACK : UInt;
	@:require(flash10_1) static final BACKQUOTE : UInt;
	@:require(flash10_1) static final BACKSLASH : UInt;
	static final BACKSPACE : UInt;
	@:require(flash10_2) static final BLUE : UInt;
	@:require(flash10_1) static final C : UInt;
	static final CAPS_LOCK : UInt;
	@:require(flash10_2) static final CHANNEL_DOWN : UInt;
	@:require(flash10_2) static final CHANNEL_UP : UInt;
	@:require(flash10_1) static final COMMA : UInt;
	@:require(flash10_1) static final COMMAND : UInt;
	static final CONTROL : UInt;
	@:require(flash10_1) static final CharCodeStrings : Array<Dynamic>;
	@:require(flash10_1) static final D : UInt;
	static final DELETE : UInt;
	static final DOWN : UInt;
	@:require(flash10_2) static final DVR : UInt;
	@:require(flash10_1) static final E : UInt;
	static final END : UInt;
	static final ENTER : UInt;
	@:require(flash10_1) static final EQUAL : UInt;
	static final ESCAPE : UInt;
	@:require(flash10_2) static final EXIT : UInt;
	@:require(flash10_1) static final F : UInt;
	static final F1 : UInt;
	static final F10 : UInt;
	static final F11 : UInt;
	static final F12 : UInt;
	static final F13 : UInt;
	static final F14 : UInt;
	static final F15 : UInt;
	static final F2 : UInt;
	static final F3 : UInt;
	static final F4 : UInt;
	static final F5 : UInt;
	static final F6 : UInt;
	static final F7 : UInt;
	static final F8 : UInt;
	static final F9 : UInt;
	@:require(flash10_2) static final FAST_FORWARD : UInt;
	@:require(flash10_1) static final G : UInt;
	@:require(flash10_2) static final GREEN : UInt;
	@:require(flash10_2) static final GUIDE : UInt;
	@:require(flash10_1) static final H : UInt;
	@:require(flash10_2) static final HELP : UInt;
	static final HOME : UInt;
	@:require(flash10_1) static final I : UInt;
	@:require(flash10_2) static final INFO : UInt;
	@:require(flash10_2) static final INPUT : UInt;
	static final INSERT : UInt;
	@:require(flash10_1) static final J : UInt;
	@:require(flash10_1) static final K : UInt;
	@:require(flash10_1) static final KEYNAME_BEGIN : String;
	@:require(flash10_1) static final KEYNAME_BREAK : String;
	@:require(flash10_1) static final KEYNAME_CLEARDISPLAY : String;
	@:require(flash10_1) static final KEYNAME_CLEARLINE : String;
	@:require(flash10_1) static final KEYNAME_DELETE : String;
	@:require(flash10_1) static final KEYNAME_DELETECHAR : String;
	@:require(flash10_1) static final KEYNAME_DELETELINE : String;
	@:require(flash10_1) static final KEYNAME_DOWNARROW : String;
	@:require(flash10_1) static final KEYNAME_END : String;
	@:require(flash10_1) static final KEYNAME_EXECUTE : String;
	@:require(flash10_1) static final KEYNAME_F1 : String;
	@:require(flash10_1) static final KEYNAME_F10 : String;
	@:require(flash10_1) static final KEYNAME_F11 : String;
	@:require(flash10_1) static final KEYNAME_F12 : String;
	@:require(flash10_1) static final KEYNAME_F13 : String;
	@:require(flash10_1) static final KEYNAME_F14 : String;
	@:require(flash10_1) static final KEYNAME_F15 : String;
	@:require(flash10_1) static final KEYNAME_F16 : String;
	@:require(flash10_1) static final KEYNAME_F17 : String;
	@:require(flash10_1) static final KEYNAME_F18 : String;
	@:require(flash10_1) static final KEYNAME_F19 : String;
	@:require(flash10_1) static final KEYNAME_F2 : String;
	@:require(flash10_1) static final KEYNAME_F20 : String;
	@:require(flash10_1) static final KEYNAME_F21 : String;
	@:require(flash10_1) static final KEYNAME_F22 : String;
	@:require(flash10_1) static final KEYNAME_F23 : String;
	@:require(flash10_1) static final KEYNAME_F24 : String;
	@:require(flash10_1) static final KEYNAME_F25 : String;
	@:require(flash10_1) static final KEYNAME_F26 : String;
	@:require(flash10_1) static final KEYNAME_F27 : String;
	@:require(flash10_1) static final KEYNAME_F28 : String;
	@:require(flash10_1) static final KEYNAME_F29 : String;
	@:require(flash10_1) static final KEYNAME_F3 : String;
	@:require(flash10_1) static final KEYNAME_F30 : String;
	@:require(flash10_1) static final KEYNAME_F31 : String;
	@:require(flash10_1) static final KEYNAME_F32 : String;
	@:require(flash10_1) static final KEYNAME_F33 : String;
	@:require(flash10_1) static final KEYNAME_F34 : String;
	@:require(flash10_1) static final KEYNAME_F35 : String;
	@:require(flash10_1) static final KEYNAME_F4 : String;
	@:require(flash10_1) static final KEYNAME_F5 : String;
	@:require(flash10_1) static final KEYNAME_F6 : String;
	@:require(flash10_1) static final KEYNAME_F7 : String;
	@:require(flash10_1) static final KEYNAME_F8 : String;
	@:require(flash10_1) static final KEYNAME_F9 : String;
	@:require(flash10_1) static final KEYNAME_FIND : String;
	@:require(flash10_1) static final KEYNAME_HELP : String;
	@:require(flash10_1) static final KEYNAME_HOME : String;
	@:require(flash10_1) static final KEYNAME_INSERT : String;
	@:require(flash10_1) static final KEYNAME_INSERTCHAR : String;
	@:require(flash10_1) static final KEYNAME_INSERTLINE : String;
	@:require(flash10_1) static final KEYNAME_LEFTARROW : String;
	@:require(flash10_1) static final KEYNAME_MENU : String;
	@:require(flash10_1) static final KEYNAME_MODESWITCH : String;
	@:require(flash10_1) static final KEYNAME_NEXT : String;
	@:require(flash10_1) static final KEYNAME_PAGEDOWN : String;
	@:require(flash10_1) static final KEYNAME_PAGEUP : String;
	@:require(flash10_1) static final KEYNAME_PAUSE : String;
	static final KEYNAME_PLAYPAUSE : String;
	@:require(flash10_1) static final KEYNAME_PREV : String;
	@:require(flash10_1) static final KEYNAME_PRINT : String;
	@:require(flash10_1) static final KEYNAME_PRINTSCREEN : String;
	@:require(flash10_1) static final KEYNAME_REDO : String;
	@:require(flash10_1) static final KEYNAME_RESET : String;
	@:require(flash10_1) static final KEYNAME_RIGHTARROW : String;
	@:require(flash10_1) static final KEYNAME_SCROLLLOCK : String;
	@:require(flash10_1) static final KEYNAME_SELECT : String;
	@:require(flash10_1) static final KEYNAME_STOP : String;
	@:require(flash10_1) static final KEYNAME_SYSREQ : String;
	@:require(flash10_1) static final KEYNAME_SYSTEM : String;
	@:require(flash10_1) static final KEYNAME_UNDO : String;
	@:require(flash10_1) static final KEYNAME_UPARROW : String;
	@:require(flash10_1) static final KEYNAME_USER : String;
	@:require(flash10_1) static final L : UInt;
	@:require(flash10_2) static final LAST : UInt;
	static final LEFT : UInt;
	@:require(flash10_1) static final LEFTBRACKET : UInt;
	@:require(flash10_2) static final LIVE : UInt;
	@:require(flash10_1) static final M : UInt;
	@:require(flash10_2) static final MASTER_SHELL : UInt;
	@:require(flash10_2) static final MENU : UInt;
	@:require(flash10_1) static final MINUS : UInt;
	@:require(flash10_1) static final N : UInt;
	@:require(flash10_2) static final NEXT : UInt;
	@:require(flash10_1) static final NUMBER_0 : UInt;
	@:require(flash10_1) static final NUMBER_1 : UInt;
	@:require(flash10_1) static final NUMBER_2 : UInt;
	@:require(flash10_1) static final NUMBER_3 : UInt;
	@:require(flash10_1) static final NUMBER_4 : UInt;
	@:require(flash10_1) static final NUMBER_5 : UInt;
	@:require(flash10_1) static final NUMBER_6 : UInt;
	@:require(flash10_1) static final NUMBER_7 : UInt;
	@:require(flash10_1) static final NUMBER_8 : UInt;
	@:require(flash10_1) static final NUMBER_9 : UInt;
	@:require(flash10_1) static final NUMPAD : UInt;
	static final NUMPAD_0 : UInt;
	static final NUMPAD_1 : UInt;
	static final NUMPAD_2 : UInt;
	static final NUMPAD_3 : UInt;
	static final NUMPAD_4 : UInt;
	static final NUMPAD_5 : UInt;
	static final NUMPAD_6 : UInt;
	static final NUMPAD_7 : UInt;
	static final NUMPAD_8 : UInt;
	static final NUMPAD_9 : UInt;
	static final NUMPAD_ADD : UInt;
	static final NUMPAD_DECIMAL : UInt;
	static final NUMPAD_DIVIDE : UInt;
	static final NUMPAD_ENTER : UInt;
	static final NUMPAD_MULTIPLY : UInt;
	static final NUMPAD_SUBTRACT : UInt;
	@:require(flash10_1) static final O : UInt;
	@:require(flash10_1) static final P : UInt;
	static final PAGE_DOWN : UInt;
	static final PAGE_UP : UInt;
	@:require(flash10_2) static final PAUSE : UInt;
	@:require(flash10_1) static final PERIOD : UInt;
	@:require(flash10_2) static final PLAY : UInt;
	static final PLAY_PAUSE : UInt;
	@:require(flash10_2) static final PREVIOUS : UInt;
	@:require(flash10_1) static final Q : UInt;
	@:require(flash10_1) static final QUOTE : UInt;
	@:require(flash10_1) static final R : UInt;
	@:require(flash10_2) static final RECORD : UInt;
	@:require(flash10_2) static final RED : UInt;
	@:require(flash10_2) static final REWIND : UInt;
	static final RIGHT : UInt;
	@:require(flash10_1) static final RIGHTBRACKET : UInt;
	@:require(flash10_1) static final S : UInt;
	@:require(flash10_2) static final SEARCH : UInt;
	@:require(flash10_1) static final SEMICOLON : UInt;
	@:require(flash10_2) static final SETUP : UInt;
	static final SHIFT : UInt;
	@:require(flash10_2) static final SKIP_BACKWARD : UInt;
	@:require(flash10_2) static final SKIP_FORWARD : UInt;
	@:require(flash10_1) static final SLASH : UInt;
	static final SPACE : UInt;
	@:require(flash10_2) static final STOP : UInt;
	@:require(flash10_1) static final STRING_BEGIN : String;
	@:require(flash10_1) static final STRING_BREAK : String;
	@:require(flash10_1) static final STRING_CLEARDISPLAY : String;
	@:require(flash10_1) static final STRING_CLEARLINE : String;
	@:require(flash10_1) static final STRING_DELETE : String;
	@:require(flash10_1) static final STRING_DELETECHAR : String;
	@:require(flash10_1) static final STRING_DELETELINE : String;
	@:require(flash10_1) static final STRING_DOWNARROW : String;
	@:require(flash10_1) static final STRING_END : String;
	@:require(flash10_1) static final STRING_EXECUTE : String;
	@:require(flash10_1) static final STRING_F1 : String;
	@:require(flash10_1) static final STRING_F10 : String;
	@:require(flash10_1) static final STRING_F11 : String;
	@:require(flash10_1) static final STRING_F12 : String;
	@:require(flash10_1) static final STRING_F13 : String;
	@:require(flash10_1) static final STRING_F14 : String;
	@:require(flash10_1) static final STRING_F15 : String;
	@:require(flash10_1) static final STRING_F16 : String;
	@:require(flash10_1) static final STRING_F17 : String;
	@:require(flash10_1) static final STRING_F18 : String;
	@:require(flash10_1) static final STRING_F19 : String;
	@:require(flash10_1) static final STRING_F2 : String;
	@:require(flash10_1) static final STRING_F20 : String;
	@:require(flash10_1) static final STRING_F21 : String;
	@:require(flash10_1) static final STRING_F22 : String;
	@:require(flash10_1) static final STRING_F23 : String;
	@:require(flash10_1) static final STRING_F24 : String;
	@:require(flash10_1) static final STRING_F25 : String;
	@:require(flash10_1) static final STRING_F26 : String;
	@:require(flash10_1) static final STRING_F27 : String;
	@:require(flash10_1) static final STRING_F28 : String;
	@:require(flash10_1) static final STRING_F29 : String;
	@:require(flash10_1) static final STRING_F3 : String;
	@:require(flash10_1) static final STRING_F30 : String;
	@:require(flash10_1) static final STRING_F31 : String;
	@:require(flash10_1) static final STRING_F32 : String;
	@:require(flash10_1) static final STRING_F33 : String;
	@:require(flash10_1) static final STRING_F34 : String;
	@:require(flash10_1) static final STRING_F35 : String;
	@:require(flash10_1) static final STRING_F4 : String;
	@:require(flash10_1) static final STRING_F5 : String;
	@:require(flash10_1) static final STRING_F6 : String;
	@:require(flash10_1) static final STRING_F7 : String;
	@:require(flash10_1) static final STRING_F8 : String;
	@:require(flash10_1) static final STRING_F9 : String;
	@:require(flash10_1) static final STRING_FIND : String;
	@:require(flash10_1) static final STRING_HELP : String;
	@:require(flash10_1) static final STRING_HOME : String;
	@:require(flash10_1) static final STRING_INSERT : String;
	@:require(flash10_1) static final STRING_INSERTCHAR : String;
	@:require(flash10_1) static final STRING_INSERTLINE : String;
	@:require(flash10_1) static final STRING_LEFTARROW : String;
	@:require(flash10_1) static final STRING_MENU : String;
	@:require(flash10_1) static final STRING_MODESWITCH : String;
	@:require(flash10_1) static final STRING_NEXT : String;
	@:require(flash10_1) static final STRING_PAGEDOWN : String;
	@:require(flash10_1) static final STRING_PAGEUP : String;
	@:require(flash10_1) static final STRING_PAUSE : String;
	@:require(flash10_1) static final STRING_PREV : String;
	@:require(flash10_1) static final STRING_PRINT : String;
	@:require(flash10_1) static final STRING_PRINTSCREEN : String;
	@:require(flash10_1) static final STRING_REDO : String;
	@:require(flash10_1) static final STRING_RESET : String;
	@:require(flash10_1) static final STRING_RIGHTARROW : String;
	@:require(flash10_1) static final STRING_SCROLLLOCK : String;
	@:require(flash10_1) static final STRING_SELECT : String;
	@:require(flash10_1) static final STRING_STOP : String;
	@:require(flash10_1) static final STRING_SYSREQ : String;
	@:require(flash10_1) static final STRING_SYSTEM : String;
	@:require(flash10_1) static final STRING_UNDO : String;
	@:require(flash10_1) static final STRING_UPARROW : String;
	@:require(flash10_1) static final STRING_USER : String;
	@:require(flash10_2) static final SUBTITLE : UInt;
	@:require(flash10_1) static final T : UInt;
	static final TAB : UInt;
	@:require(flash10_1) static final U : UInt;
	static final UP : UInt;
	@:require(flash10_1) static final V : UInt;
	@:require(flash10_2) static final VOD : UInt;
	@:require(flash10_1) static final W : UInt;
	@:require(flash10_1) static final X : UInt;
	@:require(flash10_1) static final Y : UInt;
	@:require(flash10_2) static final YELLOW : UInt;
	@:require(flash10_1) static final Z : UInt;
	@:flash.property static var capsLock(get,never) : Bool;
	@:flash.property @:require(flash10_1) static var hasVirtualKeyboard(get,never) : Bool;
	@:flash.property static var numLock(get,never) : Bool;
	@:flash.property @:require(flash10_1) static var physicalKeyboardType(get,never) : KeyboardType;
	private static function get_capsLock() : Bool;
	private static function get_hasVirtualKeyboard() : Bool;
	private static function get_numLock() : Bool;
	private static function get_physicalKeyboardType() : KeyboardType;
	static function isAccessible() : Bool;
}
