package flash.net;

@:require(flash10_1) extern final class NetStreamMulticastInfo {
	@:flash.property var bytesPushedFromPeers(get,never) : Float;
	@:flash.property var bytesPushedToPeers(get,never) : Float;
	@:flash.property var bytesReceivedFromIPMulticast(get,never) : Float;
	@:flash.property var bytesReceivedFromServer(get,never) : Float;
	@:flash.property var bytesRequestedByPeers(get,never) : Float;
	@:flash.property var bytesRequestedFromPeers(get,never) : Float;
	@:flash.property var fragmentsPushedFromPeers(get,never) : Float;
	@:flash.property var fragmentsPushedToPeers(get,never) : Float;
	@:flash.property var fragmentsReceivedFromIPMulticast(get,never) : Float;
	@:flash.property var fragmentsReceivedFromServer(get,never) : Float;
	@:flash.property var fragmentsRequestedByPeers(get,never) : Float;
	@:flash.property var fragmentsRequestedFromPeers(get,never) : Float;
	@:flash.property var receiveControlBytesPerSecond(get,never) : Float;
	@:flash.property var receiveDataBytesPerSecond(get,never) : Float;
	@:flash.property var receiveDataBytesPerSecondFromIPMulticast(get,never) : Float;
	@:flash.property var receiveDataBytesPerSecondFromServer(get,never) : Float;
	@:flash.property var sendControlBytesPerSecond(get,never) : Float;
	@:flash.property var sendControlBytesPerSecondToServer(get,never) : Float;
	@:flash.property var sendDataBytesPerSecond(get,never) : Float;
	function new(sendDataBytesPerSecond : Float, sendControlBytesPerSecond : Float, receiveDataBytesPerSecond : Float, receiveControlBytesPerSecond : Float, bytesPushedToPeers : Float, fragmentsPushedToPeers : Float, bytesRequestedByPeers : Float, fragmentsRequestedByPeers : Float, bytesPushedFromPeers : Float, fragmentsPushedFromPeers : Float, bytesRequestedFromPeers : Float, fragmentsRequestedFromPeers : Float, sendControlBytesPerSecondToServer : Float, receiveDataBytesPerSecondFromServer : Float, bytesReceivedFromServer : Float, fragmentsReceivedFromServer : Float, receiveDataBytesPerSecondFromIPMulticast : Float, bytesReceivedFromIPMulticast : Float, fragmentsReceivedFromIPMulticast : Float) : Void;
	private function get_bytesPushedFromPeers() : Float;
	private function get_bytesPushedToPeers() : Float;
	private function get_bytesReceivedFromIPMulticast() : Float;
	private function get_bytesReceivedFromServer() : Float;
	private function get_bytesRequestedByPeers() : Float;
	private function get_bytesRequestedFromPeers() : Float;
	private function get_fragmentsPushedFromPeers() : Float;
	private function get_fragmentsPushedToPeers() : Float;
	private function get_fragmentsReceivedFromIPMulticast() : Float;
	private function get_fragmentsReceivedFromServer() : Float;
	private function get_fragmentsRequestedByPeers() : Float;
	private function get_fragmentsRequestedFromPeers() : Float;
	private function get_receiveControlBytesPerSecond() : Float;
	private function get_receiveDataBytesPerSecond() : Float;
	private function get_receiveDataBytesPerSecondFromIPMulticast() : Float;
	private function get_receiveDataBytesPerSecondFromServer() : Float;
	private function get_sendControlBytesPerSecond() : Float;
	private function get_sendControlBytesPerSecondToServer() : Float;
	private function get_sendDataBytesPerSecond() : Float;
	function toString() : String;
}
