package flash.display3D;

@:native("flash.display3D.Context3DBlendFactor") extern enum abstract Context3DBlendFactor(String) {
	var DESTINATION_ALPHA;
	var DESTINATION_COLOR;
	var ONE;
	var ONE_MINUS_DESTINATION_ALPHA;
	var ONE_MINUS_DESTINATION_COLOR;
	var ONE_MINUS_SOURCE_ALPHA;
	var ONE_MINUS_SOURCE_COLOR;
	var SOURCE_ALPHA;
	var SOURCE_COLOR;
	var ZERO;
}
