package flash.text.engine;

@:native("flash.text.engine.JustificationStyle") extern enum abstract JustificationStyle(String) {
	var PRIORITIZE_LEAST_ADJUSTMENT;
	var PUSH_IN_KINSOKU;
	var PUSH_OUT_ONLY;
}
