package flash.display3D;

@:native("flash.display3D.Context3DTextureFilter") extern enum abstract Context3DTextureFilter(String) {
	var ANISOTROPIC16X;
	var ANISOTROPIC2X;
	var ANISOTROPIC4X;
	var ANISOTROPIC8X;
	var LINEAR;
	var NEAREST;
}
