package flash.display;

extern class SWFVersion {
	static final FLASH1 : UInt;
	static final FLASH10 : UInt;
	static final FLASH11 : UInt;
	static final FLASH12 : UInt;
	static final FLASH2 : UInt;
	static final FLASH3 : UInt;
	static final FLASH4 : UInt;
	static final FLASH5 : UInt;
	static final FLASH6 : UInt;
	static final FLASH7 : UInt;
	static final FLASH8 : UInt;
	static final FLASH9 : UInt;
}
