package flash.globalization;

@:native("flash.globalization.NationalDigitsType") extern enum abstract NationalDigitsType(UInt) {
	var ARABIC_INDIC;
	var BALINESE;
	var BENGALI;
	var CHAM;
	var DEVANAGARI;
	var EUROPEAN;
	var EXTENDED_ARABIC_INDIC;
	var FULL_WIDTH;
	var GUJARATI;
	var GURMUKHI;
	var KANNADA;
	var KAYAH_LI;
	var KHMER;
	var LAO;
	var LEPCHA;
	var LIMBU;
	var MALAYALAM;
	var MONGOLIAN;
	var MYANMAR;
	var MYANMAR_SHAN;
	var NEW_TAI_LUE;
	var NKO;
	var OL_CHIKI;
	var ORIYA;
	var OSMANYA;
	var SAURASHTRA;
	var SUNDANESE;
	var TAMIL;
	var TELUGU;
	var THAI;
	var TIBETAN;
	var VAI;
}
