package flash.display;

@:native("flash.display.StageQuality") extern enum abstract StageQuality(String) {
	var BEST;
	var HIGH;
	var HIGH_16X16;
	var HIGH_16X16_LINEAR;
	var HIGH_8X8;
	var HIGH_8X8_LINEAR;
	var LOW;
	var MEDIUM;
}
