package flash.net;

extern final class NetStreamInfo {
	@:flash.property var SRTT(get,never) : Float;
	@:flash.property var audioBufferByteLength(get,never) : Float;
	@:flash.property var audioBufferLength(get,never) : Float;
	@:flash.property var audioByteCount(get,never) : Float;
	@:flash.property var audioBytesPerSecond(get,never) : Float;
	@:flash.property var audioLossRate(get,never) : Float;
	@:flash.property var byteCount(get,never) : Float;
	@:flash.property var currentBytesPerSecond(get,never) : Float;
	@:flash.property var dataBufferByteLength(get,never) : Float;
	@:flash.property var dataBufferLength(get,never) : Float;
	@:flash.property var dataByteCount(get,never) : Float;
	@:flash.property var dataBytesPerSecond(get,never) : Float;
	@:flash.property var droppedFrames(get,never) : Float;
	@:flash.property var isLive(get,never) : Bool;
	@:flash.property var maxBytesPerSecond(get,never) : Float;
	@:flash.property var metaData(get,never) : Dynamic;
	@:flash.property var playbackBytesPerSecond(get,never) : Float;
	@:flash.property var resourceName(get,never) : String;
	@:flash.property var uri(get,never) : String;
	@:flash.property var videoBufferByteLength(get,never) : Float;
	@:flash.property var videoBufferLength(get,never) : Float;
	@:flash.property var videoByteCount(get,never) : Float;
	@:flash.property var videoBytesPerSecond(get,never) : Float;
	@:flash.property @:require(flash10_1) var videoLossRate(get,never) : Float;
	@:flash.property var xmpData(get,never) : Dynamic;
	function new(curBPS : Float, byteCount : Float, maxBPS : Float, audioBPS : Float, audioByteCount : Float, videoBPS : Float, videoByteCount : Float, dataBPS : Float, dataByteCount : Float, playbackBPS : Float, droppedFrames : Float, audioBufferByteLength : Float, videoBufferByteLength : Float, dataBufferByteLength : Float, audioBufferLength : Float, videoBufferLength : Float, dataBufferLength : Float, srtt : Float, audioLossRate : Float, videoLossRate : Float, ?metaData : Dynamic, ?xmpData : Dynamic, ?uri : String, ?resourceName : String, isLive : Bool = true) : Void;
	private function get_SRTT() : Float;
	private function get_audioBufferByteLength() : Float;
	private function get_audioBufferLength() : Float;
	private function get_audioByteCount() : Float;
	private function get_audioBytesPerSecond() : Float;
	private function get_audioLossRate() : Float;
	private function get_byteCount() : Float;
	private function get_currentBytesPerSecond() : Float;
	private function get_dataBufferByteLength() : Float;
	private function get_dataBufferLength() : Float;
	private function get_dataByteCount() : Float;
	private function get_dataBytesPerSecond() : Float;
	private function get_droppedFrames() : Float;
	private function get_isLive() : Bool;
	private function get_maxBytesPerSecond() : Float;
	private function get_metaData() : Dynamic;
	private function get_playbackBytesPerSecond() : Float;
	private function get_resourceName() : String;
	private function get_uri() : String;
	private function get_videoBufferByteLength() : Float;
	private function get_videoBufferLength() : Float;
	private function get_videoByteCount() : Float;
	private function get_videoBytesPerSecond() : Float;
	private function get_videoLossRate() : Float;
	private function get_xmpData() : Dynamic;
	function toString() : String;
}
