package flash.media;

extern class H264Level {
	function new() : Void;
	static final LEVEL_1 : String;
	static final LEVEL_1B : String;
	static final LEVEL_1_1 : String;
	static final LEVEL_1_2 : String;
	static final LEVEL_1_3 : String;
	static final LEVEL_2 : String;
	static final LEVEL_2_1 : String;
	static final LEVEL_2_2 : String;
	static final LEVEL_3 : String;
	static final LEVEL_3_1 : String;
	static final LEVEL_3_2 : String;
	static final LEVEL_4 : String;
	static final LEVEL_4_1 : String;
	static final LEVEL_4_2 : String;
	static final LEVEL_5 : String;
	static final LEVEL_5_1 : String;
}
