package flash.text.engine;

extern final class ElementFormat {
	@:flash.property var alignmentBaseline(get,set) : TextBaseline;
	@:flash.property var alpha(get,set) : Float;
	@:flash.property var baselineShift(get,set) : Float;
	@:flash.property var breakOpportunity(get,set) : BreakOpportunity;
	@:flash.property var color(get,set) : UInt;
	@:flash.property var digitCase(get,set) : DigitCase;
	@:flash.property var digitWidth(get,set) : DigitWidth;
	@:flash.property var dominantBaseline(get,set) : TextBaseline;
	@:flash.property var fontDescription(get,set) : FontDescription;
	@:flash.property var fontSize(get,set) : Float;
	@:flash.property var kerning(get,set) : Kerning;
	@:flash.property var ligatureLevel(get,set) : LigatureLevel;
	@:flash.property var locale(get,set) : String;
	@:flash.property var locked(get,set) : Bool;
	@:flash.property var textRotation(get,set) : TextRotation;
	@:flash.property var trackingLeft(get,set) : Float;
	@:flash.property var trackingRight(get,set) : Float;
	@:flash.property var typographicCase(get,set) : TypographicCase;
	function new(?fontDescription : FontDescription, fontSize : Float = 12, color : UInt = 0, alpha : Float = 1, ?textRotation : TextRotation, ?dominantBaseline : TextBaseline, ?alignmentBaseline : TextBaseline, baselineShift : Float = 0, ?kerning : Kerning, trackingRight : Float = 0, trackingLeft : Float = 0, ?locale : String, ?breakOpportunity : BreakOpportunity, ?digitCase : DigitCase, ?digitWidth : DigitWidth, ?ligatureLevel : LigatureLevel, ?typographicCase : TypographicCase) : Void;
	function clone() : ElementFormat;
	function getFontMetrics() : FontMetrics;
	private function get_alignmentBaseline() : TextBaseline;
	private function get_alpha() : Float;
	private function get_baselineShift() : Float;
	private function get_breakOpportunity() : BreakOpportunity;
	private function get_color() : UInt;
	private function get_digitCase() : DigitCase;
	private function get_digitWidth() : DigitWidth;
	private function get_dominantBaseline() : TextBaseline;
	private function get_fontDescription() : FontDescription;
	private function get_fontSize() : Float;
	private function get_kerning() : Kerning;
	private function get_ligatureLevel() : LigatureLevel;
	private function get_locale() : String;
	private function get_locked() : Bool;
	private function get_textRotation() : TextRotation;
	private function get_trackingLeft() : Float;
	private function get_trackingRight() : Float;
	private function get_typographicCase() : TypographicCase;
	private function set_alignmentBaseline(value : TextBaseline) : TextBaseline;
	private function set_alpha(value : Float) : Float;
	private function set_baselineShift(value : Float) : Float;
	private function set_breakOpportunity(value : BreakOpportunity) : BreakOpportunity;
	private function set_color(value : UInt) : UInt;
	private function set_digitCase(value : DigitCase) : DigitCase;
	private function set_digitWidth(value : DigitWidth) : DigitWidth;
	private function set_dominantBaseline(value : TextBaseline) : TextBaseline;
	private function set_fontDescription(value : FontDescription) : FontDescription;
	private function set_fontSize(value : Float) : Float;
	private function set_kerning(value : Kerning) : Kerning;
	private function set_ligatureLevel(value : LigatureLevel) : LigatureLevel;
	private function set_locale(value : String) : String;
	private function set_locked(value : Bool) : Bool;
	private function set_textRotation(value : TextRotation) : TextRotation;
	private function set_trackingLeft(value : Float) : Float;
	private function set_trackingRight(value : Float) : Float;
	private function set_typographicCase(value : TypographicCase) : TypographicCase;
}
