package flash.media;

extern class StageVideoAvailabilityReason {
	function new() : Void;
	static final DRIVER_TOO_OLD : String;
	static final NO_ERROR : String;
	static final UNAVAILABLE : String;
	static final USER_DISABLED : String;
	static final WMODE_INCOMPATIBLE : String;
}
